/**
  * Webcore Web Engine Abstraction Layer.
  * @file gwebcoreengine.c: Webcore Web Engine Interface Implementation
  * @version $Rev: 2122 $
  * @date $LastChangedDate: 2005-11-03 19:22:51 +0300 (Чтв, 03 Ноя 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gwebcoreengine.h"
#include <gtk/gtk.h>
#include "webi.h"


enum {
    TITLE_CHANGED,
    URL_CHANGED,
    LOADING_PROGRESS,
    START_LOADING,
    FINISHED_LOADING,
    CLOSE_WINDOW,
    POPUP_MENU,
    WINDOW_REQUEST,
    INPUT_ACTIVATE,
    INPUT_DEACTIVATE,
    LAST_SIGNAL
};

#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer


void
gtk_marshal_VOID__INT_INT_BOOL (GClosure     *closure,
                                GValue       *return_value,
                                guint         n_param_values,
                                const GValue *param_values,
                                gpointer      invocation_hint,
                                gpointer      marshal_data)
{
    typedef void (*GMarshalFunc_VOID__INT_INT_BOOL) (gpointer     data1,
            gint         arg_1,
            gint         arg_2,
            gboolean     arg_3,
            gpointer     data2);
    register GMarshalFunc_VOID__INT_INT_BOOL callback;
    register GCClosure *cc = (GCClosure*) closure;
    register gpointer data1, data2;

    g_return_if_fail (n_param_values == 4);

    if (G_CCLOSURE_SWAP_DATA (closure))
    {
        data1 = closure->data;
        data2 = g_value_peek_pointer (param_values + 0);
    }
    else
    {
        data1 = g_value_peek_pointer (param_values + 0);
        data2 = closure->data;
    }
    callback = (GMarshalFunc_VOID__INT_INT_BOOL) (marshal_data ? marshal_data : cc->callback);

    callback (data1,
              g_marshal_value_peek_int            (param_values + 1),
              g_marshal_value_peek_int            (param_values + 2),
              g_marshal_value_peek_boolean        (param_values + 3),
              data2);
}


static void g_webcore_engine_class_init (GWebcoreEngineClass *klass);
static void g_webcore_engine_init (GWebcoreEngine *gengineembed);

static void g_webcore_engine_net_stop (GtkKHTML *embed);
static void g_webcore_engine_net_start (GtkKHTML *embed);
static void g_webcore_engine_title (GtkKHTML *embed);
static void g_webcore_engine_location (GtkKHTML *embed);
static void g_webcore_engine_new_window (GtkKHTML *embed);
static void g_webcore_engine_load_url (GWebcoreEngine *embed, const gchar  *url);
//static void g_webcore_engine_progress (GtkKHTML *embed, const char *aURI, gint curprogress, gint maxprogress);
//static void g_webcore_engine_progress (GtkKHTML *embed, GtkKHTMLLoadStatus *status);
static void
g_webcore_engine_progress (Webi* engine, WebiLoadStatus* status);

static void g_webcore_engine_child_grab_focus_cb (GtkWidget *widget, GWebcoreEngine *embed);
static void g_webcore_engine_grab_focus (GtkWidget *widget);

static void g_webcore_engine_private_add (GtkContainer *container, GtkWidget *widget);
static void g_webcore_engine_private_remove (GtkContainer *container, GtkWidget *widget);


static void g_webcore_engine_base_init (gpointer g_class);
static gint g_webcore_engine_signals[LAST_SIGNAL] = {0};

static void
g_webcore_engine_base_init (gpointer g_class)
{
    static gboolean initialized = FALSE;

    if (!initialized)
    {
        g_webcore_engine_signals[URL_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_URL_CHANGED,
                        G_TYPE_WEBCORE_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GWebcoreEngineClass, location),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_webcore_engine_signals[TITLE_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_TITLE,
                        G_TYPE_WEBCORE_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GWebcoreEngineClass, title),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_webcore_engine_signals[LOADING_PROGRESS]
        = g_signal_new (G_WEBWIDGET_SIGNAL_LOADING_PROGRESS,
                        G_TYPE_WEBCORE_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GWebcoreEngineClass, progress),
                        NULL, NULL,
                        gtk_marshal_VOID__INT_INT,
                        G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);

        g_webcore_engine_signals[START_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_START_LOADING,
                        G_TYPE_WEBCORE_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GWebcoreEngineClass, load_start),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE,
                        1,
                        G_TYPE_STRING);

        g_webcore_engine_signals[FINISHED_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FINISHED_LOADING,
                        G_TYPE_WEBCORE_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET(GWebcoreEngineClass, load_stop),
                        NULL, NULL,
                        gtk_marshal_VOID__INT_INT_BOOL,
                        G_TYPE_NONE,
                        3,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_BOOLEAN);


        /* create interface signals here. */
        initialized = TRUE;
    }
}


static GtkKHTMLClass *parent_class = NULL;


static void
g_webcore_engine_destroy (GWebcoreEngine *self)
{}

static void
g_webcore_engine_load_url (GWebcoreEngine *self, const gchar *url)
{
    gtk_khtml_load_url(GTK_KHTML(self), url);
}

static GWebStatus
g_webcore_engine_load_follow_link (GWebcoreEngine *self, const gchar *url)
{
    //    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_load_url_with_css (GWebcoreEngine *self, const gchar *url, const gchar *cssfile)
{
    //    GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_webcore_engine_load_url_with_js (GWebcoreEngine *self, gboolean  enable)
{
    //GMOZ_LOG_NOT_IMP();
}

static void
g_webcore_engine_load_home_page (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    gtk_khtml_load_url(GTK_KHTML(self), "file:/usr/share/browser/home_page/Nokia_770.html");
}

static void
g_webcore_engine_stop_load (GWebcoreEngine *self)
{
    gtk_khtml_stop_load(GTK_KHTML(self));
}

static void
g_webcore_engine_reload (GWebcoreEngine *self)
{
    gtk_khtml_refresh(GTK_KHTML(self));
}

static void
g_webcore_engine_go_back (GWebcoreEngine *self, guint nsteps)
{
    gtk_khtml_go_back(GTK_KHTML(self));
}

static void
g_webcore_engine_go_forward (GWebcoreEngine *self, guint nsteps)
{
    gtk_khtml_go_forward(GTK_KHTML(self));
}

static gboolean
g_webcore_engine_can_go_back (GWebcoreEngine *self)
{
    return gtk_khtml_can_go_back(GTK_KHTML(self));
}

static gboolean
g_webcore_engine_can_go_forward (GWebcoreEngine *self)
{
    return gtk_khtml_can_go_forward(GTK_KHTML(self));
}

static void
g_webcore_engine_set_zoom_level (GWebcoreEngine *self, gint level)
{
    /*    GTimer*  timer = g_timer_new();
        g_timer_start(timer);
        gtk_webcore_set_zoom_scale (GTK_WIDGET(self), level, self->current_zoom);
        g_timer_stop(timer);
        gulong microseconds = 0;
        gdouble seconds = g_timer_elapsed(timer, NULL);
        g_timer_destroy (timer);
        printf("Zoom time = %.2f seconds\n", g_timer_elapsed(timer, NULL));
        self->current_zoom = level;*/
}

static GWebStatus
g_webcore_engine_set_zoom_level_around_point (GWebcoreEngine *self, guint x, guint y, gint level)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_compare_frames_zoom_level (GWebcoreEngine *self, gboolean *equal)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_get_image_properties (GWebcoreEngine *self, const gchar *url, gchar **file_type, guint *file_size, guint *width, guint *height)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_get_frame_properties (GWebcoreEngine *self, gchar **file_type, guint *file_size, GWebEngineEncoding *file_encoding, guint *security, gboolean *automatic_encoding)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_load_image (GWebcoreEngine *self, const gchar* url)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static gint
g_webcore_engine_get_back_window_history_list (GWebcoreEngine *self,
        GObject **history)
{
    //GMOZ_LOG_NOT_IMP();
    return 0;
}

static gint
g_webcore_engine_get_forward_window_history_list (GWebcoreEngine *self,
        GObject **history)
{
    //GMOZ_LOG_NOT_IMP();
    return 0;
}

static guint
g_webcore_engine_get_zoom_level (GWebcoreEngine *self)
{
    return self->current_zoom;
}

static void
g_webcore_engine_zoom_by_steps (GWebcoreEngine *self, guint relative_zoom, guint* legal_zoom_value)
{
    //GMOZ_LOG_NOT_IMP();
}

static void
g_webcore_engine_set_frame_zoom_level (GWebcoreEngine *self, guint level)
{
    //GMOZ_LOG_NOT_IMP();
}

static guint
g_webcore_engine_get_frame_zoom_level (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return 0;
}

static void
g_webcore_engine_set_layout_mode (GWebcoreEngine *self, GWebEngineLayoutMode mode)
{
    //GMOZ_LOG_NOT_IMP();
}

static GWebEngineLayoutMode
g_webcore_engine_get_layout_mode (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEBENGINE_LAYOUT_NORMAL;
}

static void
g_webcore_engine_set_load_images (GWebcoreEngine *self, GWebEngineImagePolicy level)
{
    //GMOZ_LOG_NOT_IMP();
}

static GWebEngineImagePolicy
g_webcore_engine_get_load_images (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEBENGINE_POLICY_ALL_IMAGES;
}

static void
g_webcore_engine_set_frame_stacking (GWebcoreEngine *self, gboolean enabled)
{
    //GMOZ_LOG_NOT_IMP();
}

static gboolean
g_webcore_engine_get_frame_stacking (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return TRUE;
}

static GWebStatus
g_webcore_engine_save_web_page (GWebcoreEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_save_current_frame (GWebcoreEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static const gchar*
g_webcore_engine_get_frame_title (GWebcoreEngine *self, gint frame_number)
{}

static gint
g_webcore_engine_get_frame_number (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return 0;
}

static GWebStatus
g_webcore_engine_save_web_page_image (GWebcoreEngine *self, const gchar *url, const gchar* directory)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_webcore_engine_select_all_text (GWebcoreEngine *self)
{}

static void
g_webcore_engine_set_language_encoding (GWebcoreEngine *self, GWebEngineEncoding encoding)
{
    /*    switch (encoding)
        {
        case G_WEBENGINE_ENCODING_UTF8:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine),"UTF-8");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_UTF16:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "UTF-16BE");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_ISO8859_1:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-1");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_ISO8859_15:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-15");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_ISO8859_2:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-2");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_ISO8859_7:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-7");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_ISO8859_9:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "ISO-8859-9");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_CP_1250:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "windows-1250");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_CP_1251:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "windows-1251");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_CP_1253:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "windows-1253");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_CP_1254:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "windows-1254");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        case G_WEBENGINE_ENCODING_KOI8_R:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine), "KOI8-R");
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        default:
            gtk_webcore_set_encoding (GTK_WIDGET(self->engine),
                                      gtk_webcore_get_encoding (GTK_WIDGET(self->engine)));
            g_webcore_engine_reload (G_WEBCORE_ENGINE(self->engine));
            break;
        }*/
}

static void
g_webcore_engine_set_frame_encoding (GWebcoreEngine *self, GWebEngineEncoding encoding)
{
    //GMOZ_LOG_NOT_IMP();
}

static gboolean
g_webcore_engine_find_text (GWebcoreEngine *self, const gchar *string, gboolean reverse, gboolean whole_word, gboolean case_sensitive, gboolean restart)
{
    return FALSE;//gtk_webcore_find_text (GTK_WIDGET(self), string, reverse, whole_word, case_sensitive, restart );
}

static void
g_webcore_engine_authenticate (GWebcoreEngine *self, gboolean ok, const gchar *username, const gchar *password)
{
    //GMOZ_LOG_NOT_IMP();
}

static GWebStatus
g_webcore_engine_cut (GWebcoreEngine *self)
{
    //    gtk_webcore_cut(GTK_WIDGET(self));
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_copy (GWebcoreEngine *self)
{
    //    gtk_webcore_copy(GTK_WIDGET(self));
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_webcore_engine_paste (GWebcoreEngine *self)
{
    //    gtk_webcore_paste(GTK_WIDGET(self));
    return G_WEB_STATUS_OK;
}

static gboolean
g_webcore_engine_can_cut (GWebcoreEngine *self)
{
    return TRUE; //gtk_webcore_can_cut(GTK_WIDGET(self));
}

static gboolean
g_webcore_engine_can_paste (GWebcoreEngine *self)
{
    return TRUE; //gtk_webcore_can_paste(GTK_WIDGET(self));
}

static gboolean
g_webcore_engine_can_copy (GWebcoreEngine *self)
{
    return TRUE; //gtk_webcore_can_copy(GTK_WIDGET(self));
}

static gboolean
g_webcore_engine_can_selecttext (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
    return TRUE;
}

static gchar*
g_webcore_engine_get_favicon_name (GWebcoreEngine *self, const gchar *favicon_folder)
{
    return NULL;
}

static GObject*
g_webcore_engine_get_engine_widget (GWebcoreEngine *self)
{
    cur_engine=self;
    /*    gtk_webcore_preference_set_int ("snav.keyCode.modifier", 0);
        gtk_webcore_preference_set_boolean ("browser.display.use_focus_colors", TRUE);
        gtk_webcore_preference_set ("browser.display.focus_text_color", "#9d0202");
        gtk_webcore_preference_set ("browser.display.focus_background_color", "#f6ff08");
        gtk_webcore_preference_set_int ("browser.display.focus_ring_width", 3);*/
    return G_OBJECT(self);
}

static void
g_webcore_engine_set_open_link_mode (GWebcoreEngine *self, gboolean in_new_window)
{
    //GMOZ_LOG_NOT_IMP();
}

static GWebStatus
g_webcore_engine_set_true_zoom_mode (GWebcoreEngine *self, gboolean status)
{
    //GMOZ_LOG_NOT_IMP();
    return G_WEB_STATUS_OK;
}

static void
g_webcore_engine_notify_plugins (GWebcoreEngine *self, gint event)
{
    //GMOZ_LOG_NOT_IMP();
}

static void
g_webcore_engine_preload_current_skin (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
}

static void
g_webcore_engine_reset (GWebcoreEngine *self)
{
    //GMOZ_LOG_NOT_IMP();
}

static void
g_webcore_engine_interface_init (GWebEngineIface *iface)
{
    iface->destroy                         = (void (*) (GWebEngine*))
            g_webcore_engine_destroy;
    iface->reset                           = (void (*) (GWebEngine*))
            g_webcore_engine_reset;
    iface->load_url                        = (void (*) (GWebEngine*, const gchar*))
            g_webcore_engine_load_url;
    iface->load_follow_link                = (GWebStatus (*) (GWebEngine*, const gchar*))
            g_webcore_engine_load_follow_link;
    iface->load_url_with_css               = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
            g_webcore_engine_load_url_with_css;
    iface->load_url_with_js                = (void (*) (GWebEngine*, gboolean))
            g_webcore_engine_load_url_with_js;
    iface->load_home_page                  = (void (*) (GWebEngine*))
            g_webcore_engine_load_home_page;
    iface->stop_load                       = (void (*) (GWebEngine*))
            g_webcore_engine_stop_load;
    iface->reload                          = (void (*) (GWebEngine*))
            g_webcore_engine_reload;
    iface->go_back                         = (void (*) (GWebEngine*, guint))
            g_webcore_engine_go_back;
    iface->go_forward                      = (void (*) (GWebEngine*, guint))
            g_webcore_engine_go_forward;
    iface->can_go_back                     = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_go_back;
    iface->can_go_forward                  = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_go_forward;
    iface->set_zoom_level                  = (void (*) (GWebEngine*, gint ))
            g_webcore_engine_set_zoom_level;
    iface->zoom_by_steps                    = (void (*) (GWebEngine*, guint, guint *))
            g_webcore_engine_zoom_by_steps;
    iface->set_zoom_level_around_point     = (GWebStatus (*) (GWebEngine*, guint, guint, gint))
            g_webcore_engine_set_zoom_level_around_point;
    iface->compare_frames_zoom_level       = (GWebStatus (*) (GWebEngine*, gboolean*))
            g_webcore_engine_compare_frames_zoom_level;
    iface->get_image_properties            = (GWebStatus (*) (GWebEngine*, const gchar*, gchar**, guint*, guint*, guint*))
            g_webcore_engine_get_image_properties;
    iface->get_frame_properties            = (GWebStatus (*) (GWebEngine*, gchar**, guint*, GWebEngineEncoding*, guint*, gboolean*))
            g_webcore_engine_get_frame_properties;
    iface->load_image                      = (GWebStatus (*) (GWebEngine*, const gchar*))
            g_webcore_engine_load_image;
    iface->get_back_window_history_list    = (gint (*) (GWebEngine*, GObject**))
            g_webcore_engine_get_back_window_history_list;
    iface->get_forward_window_history_list = (gint (*) (GWebEngine*, GObject**))
            g_webcore_engine_get_forward_window_history_list;
    iface->get_zoom_level                  = (guint (*) (GWebEngine*))
            g_webcore_engine_get_zoom_level;
    iface->set_frame_zoom_level            = (void (*) (GWebEngine*, guint))
            g_webcore_engine_set_frame_zoom_level;
    iface->get_frame_zoom_level            = (guint (*) (GWebEngine*))
            g_webcore_engine_get_frame_zoom_level;
    iface->set_layout_mode                 = (void (*) (GWebEngine*, GWebEngineLayoutMode))
            g_webcore_engine_set_layout_mode;
    iface->get_layout_mode                 = (GWebEngineLayoutMode (*) (GWebEngine*))
            g_webcore_engine_get_layout_mode;
    iface->set_load_images                 = (void (*) (GWebEngine*, GWebEngineImagePolicy))
            g_webcore_engine_set_load_images;
    iface->get_load_images                 = (GWebEngineImagePolicy (*) (GWebEngine *self))
            g_webcore_engine_get_load_images;
    iface->set_frame_stacking              = (void (*) (GWebEngine*, gboolean))
            g_webcore_engine_set_frame_stacking;
    iface->get_frame_stacking              = (gboolean (*) (GWebEngine*))
            g_webcore_engine_get_frame_stacking;
    iface->save_web_page                   = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
            g_webcore_engine_save_web_page;
    iface->save_current_frame              = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
            g_webcore_engine_save_current_frame;
    iface->get_frame_title                 = (const gchar* (*) (GWebEngine*, gint))
            g_webcore_engine_get_frame_title;
    iface->get_frame_number                = (gint (*) (GWebEngine*))
            g_webcore_engine_get_frame_number;
    iface->save_web_page_image             = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
            g_webcore_engine_save_web_page_image;
    iface->select_all_text                 = (void (*) (GWebEngine*))
            g_webcore_engine_select_all_text;
    iface->set_language_encoding           = (void (*) (GWebEngine*, GWebEngineEncoding))
            g_webcore_engine_set_language_encoding;
    iface->set_frame_encoding              = (void (*) (GWebEngine*, GWebEngineEncoding))
            g_webcore_engine_set_frame_encoding;
    iface->find_text                       = (gboolean (*) (GWebEngine*, const gchar*, gboolean, gboolean, gboolean, gboolean))
            g_webcore_engine_find_text;
    iface->authenticate                    = (void (*) (GWebEngine*, gboolean, const gchar*, const gchar*))
            g_webcore_engine_authenticate;
    iface->cut                             = (GWebStatus (*) (GWebEngine*))
            g_webcore_engine_cut;
    iface->copy                            = (GWebStatus (*) (GWebEngine*))
            g_webcore_engine_copy;
    iface->paste                           = (GWebStatus (*) (GWebEngine*))
            g_webcore_engine_paste;
    iface->can_cut                         = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_cut;
    iface->can_paste                       = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_paste;
    iface->can_copy                        = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_copy;
    iface->can_selecttext                  = (gboolean (*) (GWebEngine*))
            g_webcore_engine_can_selecttext;
    iface->get_favicon_name                = (gchar* (*) (GWebEngine*, const gchar*))
            g_webcore_engine_get_favicon_name;
    iface->get_engine_widget               = (GObject* (*) (GWebEngine*))
            g_webcore_engine_get_engine_widget;
    iface->set_open_link_mode              = (void (*) (GWebEngine*, gboolean))
            g_webcore_engine_set_open_link_mode;
    iface->set_true_zoom_mode              = (GWebStatus (*) (GWebEngine*, gboolean))
            g_webcore_engine_set_true_zoom_mode;
    iface->notify_plugins                  = (void (*) (GWebEngine*, gint))
            g_webcore_engine_notify_plugins;
    iface->preload_current_skin            = (void (*) (GWebEngine*))
            g_webcore_engine_preload_current_skin;

}

GType
g_webcore_engine_get_type (void)
{
    static GType type = 0;
    if (!type)
    {
        static GTypeInfo const object_info =
            {
                sizeof (GWebcoreEngineClass),

                (GBaseInitFunc) g_webcore_engine_base_init,
                (GBaseFinalizeFunc) NULL,

                (GClassInitFunc) g_webcore_engine_class_init,
                (GClassFinalizeFunc) NULL,
                NULL,

                sizeof (GWebcoreEngine),
                0,
                (GInstanceInitFunc) g_webcore_engine_init,
            };

        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_webcore_engine_interface_init,
                NULL,
                NULL
            };

        type = g_type_register_static (GTK_TYPE_KHTML, "GWebcoreEngine", &object_info, (GTypeFlags)0);

        g_type_add_interface_static (type,
                                     G_TYPE_WEB_ENGINE,
                                     &iface_info);
    }
    return type;
}

static void
g_webcore_engine_class_init (GWebcoreEngineClass *klass)
{
    GObjectClass *gobject_class;
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;
    GtkKHTMLClass *webcore_class;

    parent_class    = (GtkKHTMLClass *) g_type_class_peek_parent (klass);

    gobject_class   = (GObjectClass *) klass;
    object_class    = (GtkObjectClass *) klass;
    widget_class    = (GtkWidgetClass *) klass;
    webcore_class = (GtkKHTMLClass *) klass;

    webcore_class->load_stop = g_webcore_engine_net_stop;
    webcore_class->load_start = g_webcore_engine_net_start;
    webcore_class->title = g_webcore_engine_title;
    webcore_class->location = g_webcore_engine_location;
    //    webcore_class->status = g_webcore_engine_progress;

}

/* a macro for creating a new object of our type */
#define GET_WEBCORE_ENGINE_NEW (g_object_new(g_webcore_engine_get_type(), NULL))

static void
g_webcore_engine_private_add (GtkContainer *container, GtkWidget *widget)
{
    GWebcoreEngine *embed = G_WEBCORE_ENGINE (container);

    GTK_CONTAINER_CLASS(parent_class)->add
    (container, widget);

    g_object_connect (widget,
                      "signal::grab-focus", g_webcore_engine_child_grab_focus_cb, embed,
                      NULL);
}

static void
g_webcore_engine_private_remove (GtkContainer *container, GtkWidget *widget)
{
    GWebcoreEngine *embed = G_WEBCORE_ENGINE (container);

    g_object_disconnect (widget,
                         "any_signal::grab-focus", g_webcore_engine_child_grab_focus_cb, embed,
                         NULL);

    GTK_CONTAINER_CLASS(parent_class)->remove
    (container, widget);
}

gboolean testing (gpointer data)
{
    /*    GWebcoreEngine *gengineembed = (GWebcoreEngine *)data;
        g_signal_emit(gengineembed, g_webcore_engine_signals[POPUP_MENU], 0,
                      gengineembed->ctx_data.ctx_type, 
    		  gengineembed->ctx_data.xcoord, 
    		  gengineembed->ctx_data.ycoord, 
    		  gengineembed->ctx_data.linktext, 
    		  gengineembed->ctx_data.href, 
    		  gengineembed->ctx_data.imghref, 
    		  gengineembed->ctx_data.framehref);
     
     
        nsIWebBrowser *webBrowser = nsnull;
    //    gtk_moz_embed_get_nsIWebBrowser (GTK_KHTML(gengineembed), &webBrowser);
     
        nsCOMPtr<nsIDOMWindow> DOMWindow;
        webBrowser->GetContentDOMWindow(getter_AddRefs(DOMWindow));
        
        nsCOMPtr<nsISelection> selection;
        nsresult rv = DOMWindow->GetSelection(getter_AddRefs(selection));
        if (NS_SUCCEEDED(rv))
        {
    	selection->RemoveAllRanges();
        }*/
    return FALSE;
}

void
clear_ctx_data(GWebcoreEngine *gengineembed)
{
    /*    GContextData * data = &gengineembed->ctx_data;
        if (data->href)
    	g_free(data->href);
        if (data->framehref)
    	g_free(data->framehref);
        if (data->imghref)
    	g_free(data->imghref);
        if (data->linktext)
    	g_free(data->linktext);
        data->href = NULL;
        data->imghref = NULL;
        data->framehref = NULL;
        data->linktext = NULL;
        data->xcoord = NULL;
        data->ycoord = NULL;
        data->ctx_type = NULL;*/
}

gint on_button_pressed_cb(GtkWidget *button, gpointer *dom_event, GWebcoreEngine *gengineembed)
{
    /*    clear_ctx_data(gengineembed);
     
        ((nsIDOMMouseEvent*)dom_event)->GetClientX(&gengineembed->ctx_data.xcoord);
        ((nsIDOMMouseEvent*)dom_event)->GetClientY(&gengineembed->ctx_data.ycoord);
     
        gengineembed->ctx_data.ctx_type =
    	 gtk_webcore_get_context_menu_type(GTK_WIDGET(gengineembed), dom_event, &gengineembed->ctx_data);
     
        if (gengineembed->ctx_data.ctx_type & G_WEBENGINE_POPUPMENU_DOCUMENT)
        {
    	gengineembed->timer_id = g_timeout_add ( 1000, testing , gengineembed);
        }*/
    return FALSE;
}

gint on_button_released_cb(GtkWidget *button, gpointer *dom_event, GWebcoreEngine *gengineembed)
{
    /*    if (gengineembed->timer_id)
        {
          g_source_remove (gengineembed->timer_id);
          gengineembed->timer_id = 0;
          return TRUE;
        }*/
    return FALSE;
}

static void
g_webcore_engine_child_grab_focus_cb (GtkWidget *widget, GWebcoreEngine *embed)
{
    /*    GdkEvent *event;
        event = gtk_get_current_event ();
        if (!embed->_can_focus)
        {
            g_signal_stop_emission_by_name (widget, "grab-focus");
        }
        else
        {
            embed->_can_focus = FALSE;
        }*/
}

void
new_window_cb (GtkKHTML *embed, GtkKHTML **newEmbed, guint chromemask, GWebcoreEngine *gengineembed)
{
    //    g_print("embed is %p chromemask is %d\n", (void *)embed, chromemask);
    //    GObject *new_engine = g_webcore_engine_new();
    //    *newEmbed = GTK_KHTML(new_engine);
    //    g_signal_emit(gengineembed, g_webcore_engine_signals[WINDOW_REQUEST], 0, NULL, "SimpleUrl", new_engine, FALSE);
}

void
on_engine_set_cookie(Webi* engine, WebiCookie* cookie, gpointer user_data)
{
    printf("!!!!!!Set cookie FALSE by default\n");
    cookie->out_accept_cookie = FALSE;
}

static void
on_engine_status(Webi* engine, const WebiLoadStatus* status, gpointer user_data)
{
    printf("*****************Received %d of %d bytes, %d/%d files\n", status->received, status->size, status->ready, status->files);

}

static void
g_webcore_engine_init (GWebcoreEngine *gengineembed)
{
    gengineembed->current_zoom = 100;
    gengineembed->_can_focus = FALSE;
    gtk_widget_set_name(GTK_WIDGET(gengineembed), "GWebcoreEngine");

#define CONNECT_WITH_DATA(__o, __s, __h, __d)           \
  g_signal_connect (__o, __s, G_CALLBACK( __h), __d);

    //    CONNECT_WITH_DATA (engine, "location", on_engine_location, );
    //    CONNECT_WITH_DATA (engine, "title", on_engine_title, gengineembed);
    CONNECT_WITH_DATA (gengineembed, "status", on_engine_status, gengineembed);
    //    CONNECT_WITH_DATA (engine, "req-js-prompt", on_engine_req_js_prompt, gengineembed);
    //    CONNECT_WITH_DATA (engine, "req-auth-prompt", on_engine_req_auth_prompt, gengineembed);
    //    CONNECT_WITH_DATA (engine, "req-new-window", on_engine_req_new_window, gengineembed);
    //    CONNECT_WITH_DATA (engine, "show-window", on_engine_show_window, gengineembed);
    //    CONNECT_WITH_DATA (engine, "close-window", on_engine_close_window, gengineembed);
    //    CONNECT_WITH_DATA (engine, "set-window-properties", on_engine_set_window_properties, gengineembed);
    //    CONNECT_WITH_DATA (engine, "get-window-size", on_engine_get_window_size, gengineembed);
    //    CONNECT_WITH_DATA (engine, "set-window-size", on_engine_set_window_size, gengineembed);
    //    CONNECT_WITH_DATA (engine, "load-start", on_engine_load_start, gengineembed);
    //    CONNECT_WITH_DATA (engine, "load-stop", on_engine_load_stop, gengineembed);
    CONNECT_WITH_DATA (gengineembed, "set-cookie", on_engine_set_cookie, gengineembed);
    //    CONNECT_WITH_DATA (engine, "mouse-over", on_engine_mouse_over, gengineembed);

#undef CONNECT_WITH_DATA


    //    g_signal_connect (GTK_OBJECT (gengineembed), "dom_mouse_down", G_CALLBACK(on_button_pressed_cb), gengineembed);
    //    g_signal_connect (GTK_OBJECT (gengineembed), "dom_mouse_up", G_CALLBACK(on_button_released_cb), gengineembed);
    //    g_signal_connect (GTK_OBJECT (gengineembed), "new_window", G_CALLBACK(new_window_cb), gengineembed);
}

static void
g_webcore_engine_net_start (GtkKHTML *embed)
{
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_START_LOADING, (gchar*)gtk_khtml_get_location(GTK_KHTML(embed)));
}


static void
g_webcore_engine_progress (Webi* embed, WebiLoadStatus* status)
{
    printf("Received %d of %d bytes, %d/%d files\n", status->received, status->size, status->ready, status->files);
    g_return_if_fail(GTK_IS_KHTML(embed));
    g_return_if_fail(status);
    
/*    
    
    switch(status->status)
    {
    case GTK_KHTML_LOADING_START:
        g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_START_LOADING, (gchar*)gtk_khtml_get_location(GTK_KHTML(embed)));
        break;
    case GTK_KHTML_LOADING:
        g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_LOADING_PROGRESS, status->totalReceived, status->totalSize);
        break;
    case GTK_KHTML_LOADING_COMPLETE:
        break;
    case GTK_KHTML_LOADING_ERROR:
        break;
    default:
        break;
    }*/
}

static void
g_webcore_engine_net_stop (GtkKHTML *embed)
{
    GWebcoreEngine* gembed = G_WEBCORE_ENGINE(embed);

    //        if (gembed->current_zoom != 100)
    //        {
    //            gtk_webcore_set_zoom_scale(GTK_WIDGET(embed), gembed->current_zoom, 100);
    //        }
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_FINISHED_LOADING);
}

static void
g_webcore_engine_location (GtkKHTML *embed)
{
    gchar * location = (gchar*)gtk_khtml_get_location(GTK_KHTML(embed));
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_URL_CHANGED, location);
    //    g_free(location);
}

static void
g_webcore_engine_title (GtkKHTML *embed)
{
    gchar * title = (gchar*)gtk_khtml_get_title(GTK_KHTML(embed));
    g_signal_emit_by_name(embed, G_WEBWIDGET_SIGNAL_TITLE, title);
    //    g_free(title);
}



GObject*
g_webcore_engine_new_window (GWebcoreEngine *gengineembed)
{
    //    g_print("****************************g_webcore_engine_new_window\n");
}

GObject*
g_webcore_engine_new (void)
{
    /*    char *home_path;
        char *full_path;
        home_path = PR_GetEnv("HOME");
        if (!home_path)
        {
            fprintf(stderr, "Failed to get HOME\n");
            exit(1);
        }
        full_path = g_strdup_printf("%s/%s", home_path, ".webcore");
    //    gtk_moz_embed_set_profile_path(full_path, "default");
        g_free(full_path);
    */
    GWebcoreEngine *instance = G_WEBCORE_ENGINE(GET_WEBCORE_ENGINE_NEW);
    G_WEBCORE_ENGINE(instance)->engine = G_OBJECT(instance);

    printf("!!!!!!!!!!!!!!!!!Initialize proxy\n");

    webi_set_device_type(GTK_KHTML(instance), WEBI_DEVICE_TYPE_SCREEN);

    const gchar* http_proxy;


#define OSB_BROWSERGLOBAL_SETTINGS_JAVASCRIPT 1
#define OSB_BROWSERGLOBAL_SETTINGS_JAVA 0
#define OSB_BROWSERGLOBAL_SETTINGS_PLUGINS 1
#define OSB_BROWSERGLOBAL_SETTINGS_AUTOLOADIMAGES 1
#define OSB_BROWSERGLOBAL_SETTINGS_MINIMUMFONTSIZE 10
#define OSB_BROWSERGLOBAL_SETTINGS_DEFAULTFONTSIZE 16
#define OSB_BROWSERGLOBAL_SETTINGS_DEFAULTFIXEDFONTSIZE 10
#define OSB_BROWSERGLOBAL_SETTINGS_DEFAULTTEXTENCODING "ISO-8859-1"
#define OSB_BROWSERGLOBAL_SETTINGS_SERIFFONTFAMILY "serif"
#define OSB_BROWSERGLOBAL_SETTINGS_SANSSERIFFONTFAMILY "sans"
#define OSB_BROWSERGLOBAL_SETTINGS_FIXEDFONTFAMILY "fixed"
#define OSB_BROWSERGLOBAL_SETTINGS_STANDARDFONTFAMILY "times"
#define OSB_BROWSERGLOBAL_SETTINGS_CURSIVEFONTFAMILY "times"
#define OSB_BROWSERGLOBAL_SETTINGS_FANTASYFONTFAMILY "lucidabright"


    GtkKHTMLSettings t = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, WEBI_DEVICE_TYPE_HANDHELD};
    GtkKHTMLSettings *ks = &t;

    http_proxy = g_getenv ("http_proxy");
    if (!http_proxy)
        http_proxy = g_getenv ("HTTP_PROXY");

    if (!http_proxy)
    {
        /* TODO: falback to what? */
        http_proxy = "" ;
    }
    ks->http_proxy = http_proxy;
    ks->javascript_enabled = OSB_BROWSERGLOBAL_SETTINGS_JAVASCRIPT;
    ks->java_enabled = OSB_BROWSERGLOBAL_SETTINGS_JAVA;
    ks->plugins_enabled = OSB_BROWSERGLOBAL_SETTINGS_PLUGINS;
    ks->autoload_images = OSB_BROWSERGLOBAL_SETTINGS_AUTOLOADIMAGES;
    ks->minimum_font_size = OSB_BROWSERGLOBAL_SETTINGS_MINIMUMFONTSIZE;
    ks->default_font_size = OSB_BROWSERGLOBAL_SETTINGS_DEFAULTFONTSIZE;
    ks->default_fixed_font_size = OSB_BROWSERGLOBAL_SETTINGS_DEFAULTFIXEDFONTSIZE;
    ks->default_text_encoding = OSB_BROWSERGLOBAL_SETTINGS_DEFAULTTEXTENCODING;
    ks->serif_font_family = OSB_BROWSERGLOBAL_SETTINGS_SERIFFONTFAMILY;
    ks->sans_serif_font_family = OSB_BROWSERGLOBAL_SETTINGS_SANSSERIFFONTFAMILY;
    ks->fixed_font_family = OSB_BROWSERGLOBAL_SETTINGS_FIXEDFONTFAMILY;
    ks->standard_font_family = OSB_BROWSERGLOBAL_SETTINGS_STANDARDFONTFAMILY;
    ks->cursive_font_family = OSB_BROWSERGLOBAL_SETTINGS_CURSIVEFONTFAMILY;
    ks->fantasy_font_family = OSB_BROWSERGLOBAL_SETTINGS_FANTASYFONTFAMILY;

    //    plugin_path = g_strconcat (g_get_home_dir (), "/.osb-browser/plugins/", "libflashplayer.so", NULL);
    //        if (osb_browserglobal_register_plugin (global, plugin_path) == -1) {
    //	      g_warning ("couldn't register plugin: %s", plugin_path);
    //	          }
    //		      g_free(plugin_path);



    gtk_khtml_set_settings (GTK_KHTML(instance), ks);
    gtk_widget_show_all(GTK_WIDGET(instance));
    return G_OBJECT(instance);
}

GObject*
g_webcore_engine_new_with_context (gpointer context)
{
    /* FIX ME */
    return g_webcore_engine_new();
}

GObject*
g_webcore_engine_new_with_context_and_tracks (gpointer context,
        gboolean leave_tracks)
{
    /* FIX ME */
    return g_webcore_engine_new();
}
