/**
  * Webcore Web Engine Abstraction Layer.
  * @file gwebcoretransferitem.h: Webcore Transfer Item Interface Implementation
  * @version $Rev: 2101 $
  * @date $LastChangedDate: 2005-11-02 16:35:49 +0300 (Срд, 02 Ноя 2005) $
  *
  * Copyright (C) 2005 Nokia Corporation.
  *
  * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA
  *
  */

#include "gwebcoretransferitem.h"
#include <gtk/gtk.h>

static void
g_webcore_transfer_item_destroy (GWebcoreTransferItem *self)
{
    if (self->item)
    {
        gtk_object_destroy(GTK_OBJECT(self->item));
        self->item = NULL;
    }
}

static void
g_webcore_transfer_item_start (GWebcoreTransferItem *self)
{}

static void
g_webcore_transfer_item_stop (GWebcoreTransferItem *self)
{}

static GWebStatus
g_webcore_transfer_item_continue (GWebcoreTransferItem *self)
{}

static void
g_webcore_transfer_item_cancel (GWebcoreTransferItem *self)
{}

static void
g_webcore_transfer_item_reload (GWebcoreTransferItem *self)
{}

static GObject*
g_webcore_transfer_item_get_web_item (GWebcoreTransferItem *self)
{
    return self->item;
}

static const gchar*
g_webcore_transfer_item_get_url (GWebcoreTransferItem *self)
{}

static const gchar*
g_webcore_transfer_item_get_filename (GWebcoreTransferItem *self)
{}

static glong
g_webcore_transfer_item_get_total_size (GWebcoreTransferItem *self)
{}

static glong
g_webcore_transfer_item_get_progress (GWebcoreTransferItem *self)
{}

static void
g_webcore_transfer_item_interface_init (GWebTransferItemIface *iface)
{
    iface->destroy        = (void (*) (GWebTransferItem*))
                            g_webcore_transfer_item_destroy;
    iface->start          = (void (*) (GWebTransferItem*))
                            g_webcore_transfer_item_start;
    iface->stop           = (void (*) (GWebTransferItem*))
                            g_webcore_transfer_item_stop;
    iface->do_continue    = (GWebStatus (*) (GWebTransferItem*))
                            g_webcore_transfer_item_continue;
    iface->cancel         = (void (*) (GWebTransferItem*))
                            g_webcore_transfer_item_cancel;
    iface->reload         = (void (*) (GWebTransferItem*))
                            g_webcore_transfer_item_reload;
    iface->get_web_item   = (GObject* (*) (GWebTransferItem*))
                            g_webcore_transfer_item_get_web_item;
    iface->get_url        = (const gchar* (*) (GWebTransferItem*))
                            g_webcore_transfer_item_get_url;
    iface->get_filename   = (const gchar* (*) (GWebTransferItem*))
                            g_webcore_transfer_item_get_filename;
    iface->get_total_size = (glong (*) (GWebTransferItem*))
                            g_webcore_transfer_item_get_total_size;
    iface->get_progress   = (glong (*) (GWebTransferItem*))
                            g_webcore_transfer_item_get_progress;
}

static void
g_webcore_transfer_item_instance_init (GTypeInstance *instance,
                                       gpointer g_class)
{
    G_WEBCORE_TRANSFER_ITEM(instance)->item = NULL;
}

GType
g_webcore_transfer_item_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
            {
                sizeof (GWebcoreTransferItemClass),
                NULL,   /* base_init */
                NULL,   /* base_finalize */
                NULL,   /* class_init */
                NULL,   /* class_finalize */
                NULL,   /* class_data */
                sizeof (GWebcoreTransferItem),
                0,      /* n_preallocs */
                g_webcore_transfer_item_instance_init    /* instance_init */
            };
        static const GInterfaceInfo iface_info =
            {
                (GInterfaceInitFunc) g_webcore_transfer_item_interface_init,
                NULL,   /* interface_finalize */
                NULL    /* interface_data */
            };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       "GWebcoreTransferItem",
                                       &info, (GTypeFlags)0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_TRANSFER_ITEM,
                                     &iface_info);
    }
    return type;
}

/* a macro for creating a new object of our type */
#define GET_NEW (g_object_new(g_webcore_transfer_item_get_type(), NULL))

GObject*
g_webcore_transfer_item_new (void)
{
    return G_OBJECT(GET_NEW);
}

GObject*
g_webcore_transfer_item_new_with_item (GObject *item)
{
    /* FIX ME */
    return g_webcore_transfer_item_new();
}

GObject*
g_webcore_transfer_item_new_with_url (const gchar *url,
                                      const gchar *filename)
{
    /* FIX ME */
    return g_webcore_transfer_item_new();
}
